# - Find an CUDPP installation or build tree.

# When CUDPP is found, the CUDPPConfig.cmake file is sourced to setup the
# location and configuration of CUDPP.  Please read this file, or
# CUDPPConfig.cmake.in from the CUDPP source tree for the full list of
# definitions.  Of particular interest is CUDPP_USE_FILE, a CMake source file
# that can be included to set the include directories, library directories,
# and preprocessor macros.  In addition to the variables read from
# CUDPPConfig.cmake, this find module also defines
#
#  CUDPP_FOUND    - Whether CUDPP was found. If this is true, CUDPP_DIR is okay.
#
#  CUDPP_DIR      - The directory containing CUDPPConfig.cmake.  
#                  This is the only cache entry.
#  CUDPP_USE_FILE - The full path to the UseCUDPP.cmake file.
#

SET(CUDPP_DIR_STRING "directory containing CUDPPConfig.cmake.")

# Search only if the location is not already known.
IF(NOT CUDPP_DIR)
  # Get the system search path as a list.
  IF(UNIX)
    STRING(REGEX MATCHALL "[^:]+" CUDPP_DIR_SEARCH1 "$ENV{PATH}")
  ELSE(UNIX)
    STRING(REGEX REPLACE "\\\\" "/" CUDPP_DIR_SEARCH1 "$ENV{PATH}")
  ENDIF(UNIX)
  STRING(REGEX REPLACE "/;" ";" CUDPP_DIR_SEARCH2 ${CUDPP_DIR_SEARCH1})

  # Construct a set of paths relative to the system search path.
  SET(CUDPP_DIR_SEARCH "")
  FOREACH(dir ${CUDPP_DIR_SEARCH2})
    SET(CUDPP_DIR_SEARCH ${CUDPP_DIR_SEARCH} "${dir}/..")
  ENDFOREACH(dir)

  #
  # Look for an installation or build tree.
  #
  FIND_PATH(CUDPP_DIR CUDPPConfig.cmake
    # Look for an environment variable CUDPP_DIR.
    $ENV{CUDPP_DIR}

    # Look in places relative to the system executable search path.
    ${CUDPP_DIR_SEARCH}

    # Look in standard UNIX install locations.
    /usr/local/lib/CUDPP
    /usr/lib/CUDPP

    # Read from the CMakeSetup registry entries.  It is likely that
    # CUDPP will have been recently built.
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild1]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild2]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild3]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild4]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild5]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild6]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild7]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild8]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild9]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild10]

    # Help the user find it if we cannot.
    DOC "The ${CUDPP_DIR_STRING}"
  )
ENDIF(NOT CUDPP_DIR)

# If CUDPP was found, load the configuration file to get the rest of the settings.
IF(CUDPP_DIR)
  SET(CUDPP_FOUND "1")
  MESSAGE(FATAL_ERROR "CUDPP found...")
  INCLUDE("${CUDPP_DIR}/CUDPPConfig.cmake")
ELSE(CUDPP_DIR)
  SET(CUDPP_FOUND "0")
  IF(CUDPP_FIND_REQUIRED)
    MESSAGE(FATAL_ERROR "Please set CUDPP_DIR to the ${CUDPP_DIR_STRING}")
  ENDIF(CUDPP_FIND_REQUIRED)
ENDIF(CUDPP_DIR)
